/*
*
* Template Numerical Toolkit (TNT)
*
* Mathematical and Computational Sciences Division
* National Institute of Technology,
* Gaithersburg, MD USA
*
*
* This software was developed at the National Institute of Standards and
* Technology (NIST) by employees of the Federal Government in the course
* of their official duties. Pursuant to title 17 Section 105 of the
* United States Code, this software is not subject to copyright protection
* and is in the public domain. NIST assumes no responsibility whatsoever for
* its use by other parties, and makes no guarantees, expressed or implied,
* about its quality, reliability, or any other characteristic.
*
*/

#ifndef TNT_ARRAY1D_UTILS_H
#define TNT_ARRAY1D_UTILS_H

#include <cstdlib>
#include <cassert>

namespace TNT
{


template <class T>
std::ostream& operator<<(std::ostream &s, const Array1D<T> &A)
{
    int N=A.dim1();

#ifdef TNT_DEBUG
	s << "addr: " << (void *) &A[0] << "\n";
#endif
    s << N << "\n";
    for (int j=0; j<N; j++)
    {
       s << A[j] << "\n";
    }
    s << "\n";

    return s;
}

template <class T>
std::istream& operator>>(std::istream &s, Array1D<T> &A)
{
	int N;
	s >> N;

	Array1D<T> B(N);
	for (int i=0; i<N; i++)
		s >> B[i];
	A = B;
	return s;
}



//------------------------- modified bt JTB - if not same size, return intersection
template <class T>
Array1D<T> operator+(const Array1D<T> &A, const Array1D<T> &B){
	int n = min(A.dim(), B.dim());
	Array1D<T> C(n);
	for( int i=0; i<n; i++ ) 
		C[i] = A[i] + B[i];
	return C;
	}
	
template <class T>
Array1D<T> operator-(const Array1D<T> &A, const Array1D<T> &B){
	int n = min(A.dim(), B.dim());
	Array1D<T> C(n);
	for( int i=0; i<n; i++ ) 
		C[i] = A[i] - B[i];
	return C;
	}
	
template <class T>
Array1D<T> operator*(const Array1D<T> &A, const Array1D<T> &B){
	int n = min(A.dim(), B.dim());
	Array1D<T> C(n);
	for( int i=0; i<n; i++ ) 
		C[i] = A[i] * B[i];
	return C;
	}
	
template <class T>
Array1D<T> operator/(const Array1D<T> &A, const Array1D<T> &B){
	int n = min(A.dim(), B.dim());
	Array1D<T> C(n);
	for( int i=0; i<n; i++ ) 
		if( B[i] != 0 ) 
			C[i] = A[i] / B[i];
		else 
			C[i]=0;
	return C;
	}

//------------------------- modified by JTB - if not same size, operate on intersection
template <class T>
Array1D<T>& operator+=( Array1D<T> &A, const Array1D<T> &B ){
	int n = min(A.dim(), B.dim());
	for( int i=0; i<n; i++ )
		A[i] += B[i];
	return A;
	}

template <class T>
Array1D<T>& operator-=( Array1D<T> &A, const Array1D<T> &B ){
	int n = min(A.dim(), B.dim());
	for( int i=0; i<n; i++ )
		A[i] -= B[i];
	return A;
	}
	
template <class T>
Array1D<T>& operator*=( Array1D<T> &A, const Array1D<T> &B ){
	int n = min(A.dim(), B.dim());
	for( int i=0; i<n; i++ )
		A[i] *= B[i];
	return A;
	}
	
template <class T>
Array1D<T>& operator/=( Array1D<T> &A, const Array1D<T> &B ){
	int n = min(A.dim(), B.dim());
	for( int i=0; i<n; i++ )
		if( B[i] != 0 ) 
			A[i] /= B[i];
		else 
			A[i]=0;
	return A;
	}

template <class T>
T Sum( int iFirst, int iLast, T * A ) { 
	T dSum=0.0;
	for( int i=iFirst; i<=iLast; ++i )
		dSum += A[i];
	return dSum;
	}

template <class T>
T SumProduct( int iDim, T * A,  T * B ) { 
	T dSum=0.0;
	for( int i=0; i<iDim; ++i )
		dSum += A[i] * B[i];
	return dSum;
	}

template <class T>
void veccopy( int iDim, T * pdSrc,  T * pdDest ) { 
	// memcpy( pdDest, pdSrc, iDim*sizeof(T) )
	for( int i=0; i<iDim; ++i )
		pdDest[i]=pdSrc[i];
	}

template <class T>
void vec_axplusb( int iDim, T a, T * x,  T * b, T * y ) { 
	for( int i=0; i<iDim; ++i )
		y[i] = a * x[i] + b[i];
	}



#ifdef _DEBUG
template <class T>
void OutputDebugVector( const Array1D<T> & a, const char * pchName ) { 
	char buffer[256];
	strcpy( buffer, pchName );
	sprintf( buffer+strlen(buffer), "\t{" );
	for( int j=0; j<a.dim() && j<12; ++j ) 
		sprintf( buffer+strlen(buffer), " %.5f", double(a[j]) );
	if( a.dim()>=12 ) 
		strcat( buffer, "..." );
	strcat( buffer, "}\n" );
	OutputDebugString( buffer );
	}
	
void OutputDebugInts( const char * pch, int iCount, int i0, int i1, int i2, int i3, int i4, 
						int i5, int i6, int i7, int i8, int i9 ) { 
	char buffer[1000];
	sprintf( buffer+strlen(buffer), "%s", pch );
	if( iCount>0 ) 
		sprintf( buffer, "%d ", i0 );
	if( iCount>1 ) 
		sprintf( buffer+strlen(buffer), "%d ", i1 );
	if( iCount>2 ) 
		sprintf( buffer+strlen(buffer), "%d ", i2 );
	if( iCount>3 ) 
		sprintf( buffer+strlen(buffer), "%d ", i3 );
	if( iCount>4 ) 
		sprintf( buffer+strlen(buffer), "%d ", i4 );
	if( iCount>5 ) 
		sprintf( buffer+strlen(buffer), "%d ", i5 );
	if( iCount>6 ) 
		sprintf( buffer+strlen(buffer), "%d ", i6 );
	if( iCount>7 ) 
		sprintf( buffer+strlen(buffer), "%d ", i7 );
	if( iCount>8 ) 
		sprintf( buffer+strlen(buffer), "%d ", i8 );
	if( iCount>9 ) 
		sprintf( buffer+strlen(buffer), "%d ", i9 );
	sprintf( buffer+strlen(buffer), "\n" );
	OutputDebugString( buffer );
	}
#endif





} // namespace TNT

#endif
